﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows.Automation;
using System.Windows;
using System.Runtime.InteropServices;


namespace UITestClient
{
    [StructLayout(LayoutKind.Sequential)]
    public struct HARDWAREINPUT
    {
        public uint msg;
        public ushort paramL;
        public ushort paramH;
    }

    [StructLayout(LayoutKind.Sequential)]
    public struct KEYBDINPUT
    {
        public ushort virtualKeyCode;
        public ushort scanCode;
        public uint flags;
        public uint time;
        public IntPtr extraInfo;
    }

    [StructLayout(LayoutKind.Sequential)]
    public struct MOUSEINPUT
    {
        public int dx;
        public int dy;
        public uint mouseData;
        public uint flags;
        public uint time;
        public IntPtr extraInfo;
    }


    [StructLayout(LayoutKind.Explicit)]
    public struct InputTypeUnion
    {
        // Fields
        [FieldOffset(0)]
        public HARDWAREINPUT hi;
        [FieldOffset(0)]
        public KEYBDINPUT ki;
        [FieldOffset(0)]
        public MOUSEINPUT mi;
    }

    [StructLayout(LayoutKind.Sequential)]
    public struct INPUT
    {
        public uint type;
        public InputTypeUnion data;
    }

    class Clicker
    {
        [DllImport("user32.dll")]
        static extern uint SendInput(uint inputCount, ref INPUT inputs, int inputSize);

        static public void Click(Point point)
        {            
            MovePointTo (point.X,point.Y);            
            Click();
        }

        static public Point GetClickPoint(AutomationElement ele)
        {            
            Point clickablePt;
            if(ele.TryGetClickablePoint(out clickablePt))
            {
                return clickablePt;
            }
            var boundingRect = ele.Current.BoundingRectangle;           
            return new Point(boundingRect.X + (boundingRect.Width / 2), boundingRect.Y + (boundingRect.Height / 2));            
        }

        static void MovePointTo(double absX, double absY)
        {
            INPUT input = new INPUT();
            var virtualScreen = System.Windows.Forms.SystemInformation.VirtualScreen;
            absX = (((absX - virtualScreen.X) + 0.5) * 65536.0) / ((double)virtualScreen.Width);
            absY = (((absY - virtualScreen.Y) + 0.5) * 65536.0) / ((double)virtualScreen.Height);
            input.type = 0;
            input.data.mi.dx = (int)absX;
            input.data.mi.dy = (int)absY;
            input.data.mi.flags = 0xc001;
            SendInput(1, ref input, Marshal.SizeOf(input));
        }

        static void MouseDown()
        {
            INPUT input = new INPUT();
            input.type = 0;
            input.data.mi.flags = 2; //DOWN
            input.data.mi.mouseData |= 1;
            SendInput(1, ref input, Marshal.SizeOf(input));
        }

        static void MouseUp()
        {
            INPUT input = new INPUT();
            input.type = 0;
            input.data.mi.flags = 4; //UP
            input.data.mi.mouseData |= 1;
            SendInput(1, ref input, Marshal.SizeOf(input));
        }

        static void Click()
        {
            MouseDown();
            MouseUp();
        }
    }   
}
