﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows.Shapes;
using System.Windows.Forms;
using System.Windows.Forms.Integration;
using System.Windows.Automation;
using System.Windows.Automation.Provider;
using System.Runtime.InteropServices;

namespace WPFUIAutoTarget
{
    [ComVisible(true)]
    public class MyForm : System.Windows.Forms.Form, IRawElementProviderSimple
    {
        private ValuePattern vp = new ValuePattern();
        private Timer timer = new Timer();
        private string curentNameProperty = string.Empty;

        public MyForm()
        {
            timer.Interval = 500;
            timer.Tick += new EventHandler(timer_Tick);
            timer.Enabled = true;

            this.Name = "WinFormWindow";
            this.Text = "ServerUIAFormDemo";

            Button btn = new Button();
            btn.Text = "This is a WinForm Button";
            btn.Name = "button1";
            this.Controls.Add(btn);

            curentNameProperty = string.Format("{0} {1}", this.Name, DateTime.Now.ToLongTimeString());
        }

        void timer_Tick(object sender, EventArgs e)
        {
            string newNameProperty=string.Format("{0} {1}", this.Name, DateTime.Now.ToLongTimeString());
            AutomationInteropProvider.RaiseAutomationPropertyChangedEvent(this, new AutomationPropertyChangedEventArgs(AutomationElement.NameProperty, curentNameProperty, newNameProperty));
            curentNameProperty = newNameProperty;
        }

        protected override void WndProc(ref Message m)
        {

            const int WM_GETOBJECT = 0x003D;
            if ((m.Msg == WM_GETOBJECT) && (m.LParam.ToInt32() ==
                AutomationInteropProvider.RootObjectId))
            {

                m.Result = AutomationInteropProvider.ReturnRawElementProvider(
                        this.Handle, m.WParam, m.LParam,
                        (IRawElementProviderSimple)this);
                return;

            }
            base.WndProc(ref m);
        } 

        public Object GetPatternProvider(int patternId)
        {

            if (patternId == ValuePatternIdentifiers.Pattern.Id)
            {
                //Create and return ValuePattern object
                return vp;
            }
            else
            {
                return null;
            }
        }

        //This function handles all the UIA Property reqeust
        public Object GetPropertyValue(int propertyId)
        {

            if (propertyId == AutomationElementIdentifiers.NameProperty.Id)
            {
                return curentNameProperty;
            }

            else if (propertyId == AutomationElementIdentifiers.NativeWindowHandleProperty.Id)
            {
                return this.Handle;
            }

            else if (propertyId == AutomationElementIdentifiers.AutomationIdProperty.Id)
            {
                return this.Name;
            }

            else if (propertyId == AutomationElementIdentifiers.ClassNameProperty.Id)
            {
                return "RootButtonControlClass";
            }

            else if (propertyId == AutomationElementIdentifiers.ControlTypeProperty.Id)
            {
                return ControlType.Window.Id;
            }

            else if (propertyId == AutomationElementIdentifiers.IsContentElementProperty.Id)
            {
                return false;
            }

            else if (propertyId == AutomationElementIdentifiers.IsControlElementProperty.Id)
            {
                return true;
            }
            else
            {
                return AutomationElement.NotSupported;
            }

        }

        public IRawElementProviderSimple HostRawElementProvider
        {
            get
            {
                return AutomationInteropProvider.HostProviderFromHandle(this.Handle);
            }
        }

        public ProviderOptions ProviderOptions
        {
            get
            {
                //Indicate this is server side implementation
                return ProviderOptions.ServerSideProvider;
            }
        }
    }

    [ComVisible(true)]
    public class ValuePattern : IValueProvider
    {
        public bool IsReadOnly { get { return true; } }
        public string Value
        {
            get
            {
                //Return current time as value pattern’s value
                return DateTime.Now.ToLongTimeString();
            }
        }
        public void SetValue(string value) { return; }
    }
}
