﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using System.Windows.Automation.Peers;
using  System.Windows.Automation.Provider;

namespace WPFUIAutoTarget
{
    /// <summary>
    /// Interaction logic for SelfDrawControl.xaml
    /// </summary>
    public partial class SelfDrawControl : UserControl
    {
        public SelfDrawControl()
        {
            InitializeComponent();
        }

        protected override void OnRender(DrawingContext dc)
        {
            dc.DrawRectangle(Brushes.Blue, new Pen(Brushes.Blue, 10), new Rect(new Point(0, 0), new Point(RenderSize.Width / 2, RenderSize.Height)));
            dc.DrawRectangle(Brushes.Black, new Pen(Brushes.Black, 10), new Rect(new Point(RenderSize.Width / 2, 0), new Point(RenderSize.Width, RenderSize.Height)));
        
        }

        protected override void OnMouseLeftButtonDown(MouseButtonEventArgs e)
        {
            Point point = e.GetPosition(this);
            if (point.X > 0 && point.X < RenderSize.Width / 2)
            {
                DoClick("blue");
            }
            else
            {
                DoClick("black");
            }
            base.OnMouseLeftButtonDown(e);
        }

        internal void DoClick(string color)
        {
            MessageBox.Show(color);
        }

        protected override System.Windows.Automation.Peers.AutomationPeer OnCreateAutomationPeer()
        {
            var peer = new SelfDrawControlAutomationPeer(this);
            peer.InvalidatePeer();
            return peer;
        }       
    }

    public class SelfDrawControlAutomationPeer :UserControlAutomationPeer
    {
        SelfDrawControl target;
        List<AutomationPeer> children = null;
        public SelfDrawControlAutomationPeer(SelfDrawControl target):base(target)
        {
            this.target = target;
        }

        protected override AutomationControlType GetAutomationControlTypeCore()
        {
            return AutomationControlType.Window;
        }

        protected override string GetClassNameCore()
        {
            return target.GetType().ToString();
        }

        /*
         * 特别注意， 生成ChildrenCore的时候务必维护父子关系
         * UI Testclient 可能在不同child parent之间遍历
         * 务必保证childpeer.Parent = parent.Children[n]
         * 否则会带来各种意外，这是在实现自定义Peer时候的最大陷阱
        */ 

        protected override List<AutomationPeer> GetChildrenCore()
        {
            //return null;

            if (children == null)
            {
                children = new List<AutomationPeer>();
                SelfDrawControlElementAutomationPeer bluepeer = new SelfDrawControlElementAutomationPeer(target, this, "blue");
                SelfDrawControlElementAutomationPeer redpeer = new SelfDrawControlElementAutomationPeer(target, this, "red");
                children.Add(bluepeer);
                children.Add(redpeer);
            }

            return children;
        }

    }

    public class SelfDrawControlElementAutomationPeer : AutomationPeer, IInvokeProvider, IValueProvider
    {
        private string color;
        SelfDrawControl target;
        SelfDrawControlAutomationPeer parentPeer;

        public SelfDrawControlElementAutomationPeer(SelfDrawControl target, SelfDrawControlAutomationPeer parentPeer, string color)             
        {
            this.color = color;
            this.target = target;
            this.parentPeer = parentPeer;

            var o=this.GetParent();
        }

        public void Invoke()
        {
            target.DoClick(color);
        }

        public bool IsReadOnly { get { return true; } }
        public string Value { get { return color; } }
        public void SetValue(string value) { }

        protected override string GetAcceleratorKeyCore()
        {
            return string.Empty;
        }
       
   
        protected override string GetAccessKeyCore()
        {
            return string.Empty;
        }
      
        protected override AutomationControlType GetAutomationControlTypeCore()
        {
            return AutomationControlType.Window;
        }
      
        protected override string GetAutomationIdCore()
        {           
            return target.Name+"-"+color;
        }
    
        protected override Rect GetBoundingRectangleCore()
        {

            Rect parentRect=parentPeer.GetBoundingRectangle();

            if(color=="blue")
            {
                return new Rect(new Point(parentRect.X, parentRect.Y), new Point(parentRect.X + target.RenderSize.Width / 2, parentRect.Y+target.RenderSize.Height));
            }
            if(color=="red")
            {
                return new Rect(new Point(parentRect.X + target.RenderSize.Width / 2, parentRect.Y), new Point(parentRect.X + target.RenderSize.Width, parentRect.Y + target.RenderSize.Height));
            }
            return Rect.Empty;
        }
     
        protected override List<AutomationPeer> GetChildrenCore()
        {
            return null;
        }
   
        protected override string GetClassNameCore()
        {
            return "SelfDrawControlInnerGraphic";
        }
      
        protected override Point GetClickablePointCore()
        {
            return new Point(0,0);
        }
  
        protected override string GetHelpTextCore()
        {
            return "This is my HelpText";
        }
      
        protected override string GetItemStatusCore()
        {
            return "Status is active";
        }
    
        protected override string GetItemTypeCore()
        {
            return "This is my item type";
        }
     
        protected override AutomationPeer GetLabeledByCore()
        {
            return this;
        }
      
        protected override string GetNameCore()
        {
            return "Name is :"+color;
        }
       
        protected override AutomationOrientation GetOrientationCore()
        {
            return AutomationOrientation.None;
        }
     
        public override object GetPattern(PatternInterface patternInterface)
        {
            if(patternInterface== PatternInterface.Invoke || patternInterface== PatternInterface.Value)
            {
                return this;
            }
            return null;
        }
     
        protected override bool HasKeyboardFocusCore()
        {
            return false;
        }
       
        protected override bool IsContentElementCore()
        {
            return true;
        }
     
        protected override bool IsControlElementCore()
        {
            return true;
        }
  
        protected override bool IsEnabledCore()
        {
            return true;
        }

        protected override bool IsKeyboardFocusableCore()
        {
            return false;
        }
   
        protected override bool IsOffscreenCore()
        {
            return false;
        }
      
        protected override bool IsPasswordCore()
        {
            return false;
        }
    
        protected override bool IsRequiredForFormCore()
        {
            return false;
        }
       
        protected override void SetFocusCore()
        {
        }
    }
    
}
